/*                                                                        *
*    File is Copyright  2013 CSR Technology Inc.  All rights reserved.   *
*    No implied or expressed warranties are granted.                      *
*                                                                         *
*    This Software is protected by United States copyright laws and       *
*    international treaties.  You may not reverse engineer, decompile     *
*    or disassemble this Software.                                        *
*                                                                         *
*    WARNING:                                                             *
*    This Software contains CSR Technology Inc.s confidential and        *
*    proprietary information. UNAUTHORIZED COPYING, USE, DISTRIBUTION,    *
*    PUBLICATION, TRANSFER, SALE, RENTAL OR DISCLOSURE IS PROHIBITED      *
*    AND MAY RESULT IN SERIOUS LEGAL CONSEQUENCES.  Do not copy this      *
*    Software without CSR Technology, Inc.s  express written             *
*    permission. Use of any portion of the contents of this Software      *
*    is subject to and restricted by your signed written agreement with   *
*    CSR Technology, Inc.                                                 *
*                                                                         *
***************************************************************************
*
* MODULE:  ROM Patch Application
*
* FILENAME:  ROMPatchSettings.h
*
* DESCRIPTION:This file contains the defines and function declaration related ROM Patch 
*                         Transfer and verification
*
***************************************************************************
*
*  Keywords for Perforce.  Do not modify.
*
*  $File: //core/tools/ROMPatchApplication/src/ROMPatchSettings.h $
*
*  $DateTime: 2015/09/28 18:35:52 $
*
*  $Revision: #11 $
*
***************************************************************************/
#include <windows.h>

#ifndef _ROM_PATCHSETTINGS_H
#define _ROM_PATCHSETTINGS_H

/* Max allowable Patch size is 64K - 1 */
#define MAX_PATCH_DATA_SIZE  (65535) 

/* 4e ROM versions since 0x1A use CRC32 instead of CRC16 */
#define ROMVERSIONCODE_USE_CRC32  0x1A

#define MIN(a,b)  (((a) < (b)) ? (a) : (b))

/* Contains 1 byte for MID, 1 byte for SID, and 2 bytes for SeqNo */
#define MID_PM_LOAD_FIXED_SIZE (4)

#define PM_MAX_BUFFER_SIZE  1024

/* 0xA0, 0xA2, 2 bytes Payload Length, 2 byte for Checksum, 0xB0, and 0xB3 */
#define PM_ENVELOPE_SIZE    8

/* 1024 - 8 = 1016, removes A0A2, MsgSize, and B0B3 */
#define PM_MAX_INPUT_PAYLOAD_SIZE (PM_MAX_BUFFER_SIZE - PM_ENVELOPE_SIZE)

/* 1016 - 4 = 1012, removes MID, SID, and Seqno */
#define PM_MAX_LOADDATA_SIZE (PM_MAX_INPUT_PAYLOAD_SIZE - MID_PM_LOAD_FIXED_SIZE)

#define PATCH_FILE_CONTENT_START_OFFSET     (16)
#define PATCH_FILE_HEADERINFO_SIZE          (16)
#define PATCHLOAD_HDRSZ (PATCH_FILE_HEADERINFO_SIZE - 2)

UINT8 gPatchFileCRCSize;

/* Prompt notifies host of chip, ROM and patch versions */
typedef struct
{
   UINT16 ChipId;
   UINT16 SiliconVersion;
   UINT16 ROMVersionCode;
   UINT16 PatchVersionCode;
} tPMPatchManagerPrompt;

/** Native format of patch file info record */
typedef struct
{
   UINT8  Head1;
   UINT8  Head2;
   UINT16 ChipId;
   UINT16 SiliconVersion;
   UINT16 ROMVersionCode;
   UINT16 PatchRevisionCode;
   UINT32 PatchDataBaseAddress;     /* Don't know if this is necessary */
   UINT16 PatchDataLength;          /* Size of patch data in PD2 */
   UINT16 PatchRAMStartOffset;      /* Start offset of patch ram code in PatchData array */ 
   UINT16 PatchDataNOVLSize;        /* Size of non-overlay section */
   UINT32 PatchDataCRC;             /* CRC of patch payload of initial PD2 file */
   UINT32 PatchDataOVLCRC;          /* CRC of overlay section only */
   UINT32 PatchDataNOVLCRC;         /* CRC of non-overlay section only */
   UINT8  PatchData[MAX_PATCH_DATA_SIZE];
} tPMPatchFileInfo;

typedef enum
{
   PM_LOAD_IDLE,
   PM_LOAD_INPROGRESS,
   PM_LOAD_DONE
} PMPatchLoadState;

/** Source-data bookkeeping for piecewise patch file loading. This
 *  describes the next block of data that will be sent.            */
typedef struct
{
   UNINT seq;           /* outgoing sequence number */
   UNINT offs;          /* byte offset into patch data array */
   UNINT len;           /* byte length of current segment */
   UNINT hlen;          /* header byte length of current segment */
   UNINT total;         /* number of bytes in entire patch data array */
   BOOL  ack_rcvd;      /* flag used to indicate ack was rcvd */
} tPMPatchFileState;

/** Overall states for patch file, transfer and results */
typedef struct
{
   PMPatchLoadState loadstate;      /* Describes patch state */
   tPMPatchFileInfo fileinfo;       /* Describes patch file to be loaded */
   tPMPatchFileState filestate;     /* Used for patch loading. */
   tPMPatchManagerPrompt pm_prompt; /* Saved last prompt message */
   BOOL  ValidLoadInProgress;       /* Set when patch file is going to loaded */
} tPMPatchState;

/* SSB 0xB2 0x90 message from Rx */
typedef struct
{
   UINT16 ChipId;
   UINT16 SiliconVersion;
   UINT16 ROMVersionCode;
   UINT16 PatchVersionCode;
} tSIRF_MSG_SSB_PM_PROMPT_OUT;

/* SSB 0xB2 0x91 message from Rx */
typedef struct tSIRF_MSG_SSB_PM_ACK_OUT_tag
{
   UINT16 MessageSeqNo;        /* Message Seq No Acknowledged */
   UINT8  MessageIdAck;        /* Id of original message */
   UINT8  AckStatus;           /* Result code */
} tSIRF_MSG_SSB_PM_ACK_OUT;

/* PM AckStatus utilizes the 2 least significant bits. */
#define MESSAGE_RECEIVED  0x02
#define OPERATION_SUCCESS 0x01
#define OPERATION_FAILED  0x00

/* Reason for Operation Failure */
#define PATCH_RAM_CRC_FAILURE   0x04
#define DEVICE_OPEN_ERROR       0x08
#define DEVICE_CLOSE_ERROR      0x10
#define DEVICE_WRITE_ERROR      0x20
#define DEVICE_READ_ERROR       0x40
#define INCOMPLETE_PATCH        0x80

/* Mask to extract Op Failure bits */
#define PM_MSG_OP_FAIL_MASK       0xFC

/* Mask to extract Rx and Op Flag bits */
#define PM_MSG_RX_OP_MASK         0x03

/* Msg Rcvd Operation Failed */
#define PM_MSG_RX_OK_OP_FAILURE   ((MESSAGE_RECEIVED)|(OPERATION_FAILED))
/* Msg Rcvd Operation Successful */
#define PM_MSG_RX_OK_OP_SUCCESS   ((MESSAGE_RECEIVED)|(OPERATION_SUCCESS))

#define NUM_CONFIG      8
#define MAX_CONFIG_SIZE _MAX_PATH

#define MAX_NUM_TRIALS  (3)

/* Input Configuration related defines */
#define UART 0
#define SPI   1
#define OSP_PROTOCOL   0
#define NMEA_PROTOCOL 1

typedef struct 
{
    UINT32 portType;                      /* 0 = UART, 1 = SPI */
    SINT8 portName[MAX_CONFIG_SIZE];      /* 1 = "COM1", 2 = "COM2", */
    UINT32 initialRxBaudRate;             /* e.g. 38400 */
    UINT32 initialRxProtocol;             /* 0=OSP, 1 = NMEA */
    UINT32 patchBaudRate;                 /* patching baud rate */
    BOOL CCKIOProtoFlag;                  /* TRUE when CCK IO Protocol or Baud Rate or Uart Flow Control */
                                          /* or power mode defaults have changed else this field is set  */
                                          /* to FALSE  */
    SINT8 patchFilePath[MAX_CONFIG_SIZE]; /* Patch File Path and Name */
    SINT8 logFilePath[MAX_CONFIG_SIZE];   /* Log File Path and Name */
}tInputConfig; 

typedef struct
{
    UINT8  LP_State;
    UINT8  LP_DutyCycle;
    SINT16 LP_OnTime;
}RcvrParamSS_1;

RcvrParamSS_1 RcvrParamSS1;

typedef struct
{
    UINT8  LP_PushToFix;
    LONG  LP_OnTime;
    LONG  LP_Interval;
    UINT8  LP_PwrCyclingEnabled;
    ULONG LP_MaxAcqTime;
    ULONG LP_MaxOffTime;
}RcvrParamSS_2;

RcvrParamSS_2 RcvrParamSS2;
RcvrParamSS_2 SavedRcvrParamSS2;

/* External Function Declaration */
SINT32 ReadBinaryFile(SINT8* strPath, UINT8 **ppData, LONG *pSize);
SINT32 ProcessPatchFile(SINT8 *patchFileName);
SINT32 RestoreRxSettings(void);
SINT32 HandleLPModeScenario(void);
SINT32 RestoreLPModeSettings(void);


#endif /* _ROM_PATCHSETTINGS_H */

