/*                                                                        *
*    File is Copyright  2013 CSR Technology Inc.  All rights reserved.   *
*    No implied or expressed warranties are granted.                      *
*                                                                         *
*    This Software is protected by United States copyright laws and       *
*    international treaties.  You may not reverse engineer, decompile     *
*    or disassemble this Software.                                        *
*                                                                         *
*    WARNING:                                                             *
*    This Software contains CSR Technology Inc.s confidential and        *
*    proprietary information. UNAUTHORIZED COPYING, USE, DISTRIBUTION,    *
*    PUBLICATION, TRANSFER, SALE, RENTAL OR DISCLOSURE IS PROHIBITED      *
*    AND MAY RESULT IN SERIOUS LEGAL CONSEQUENCES.  Do not copy this      *
*    Software without CSR Technology, Inc.s  express written             *
*    permission. Use of any portion of the contents of this Software      *
*    is subject to and restricted by your signed written agreement with   *
*    CSR Technology, Inc.                                                 *
*                                                                         *
***************************************************************************
*
* MODULE:  ROM Patch Application
*
* FILENAME:  CommInterface.c
*
* DESCRIPTION:This file contains the defenition of communication API's. These API's 
*                        internally calls the communication driver API's 
*
***************************************************************************
*
*  Keywords for Perforce.  Do not modify.
*
*  $File: //core/tools/ROMPatchApplication/src/abstraction/CommInterface.c $
*
*  $DateTime: 2013/02/27 23:48:03 $
*
*  $Revision: #3 $
*
***************************************************************************/

#include "basicTypes.h"
#include "CommInterface.h"
#include "serialDriver.h"
#include "logs.h"
#include "../errorCodes.h"

/*
* Function : Comm_DevInit
* Purpose  : Dev Initialization routine. The function finds the host port id and calls 
*            the device initialization routune from the ioDriverList array
* Inputs   : void *pParams
* Outputs  : None
* Returns  :E_GENERIC__FAIL For failure
E_GENERIC__OK,  For success
*/

T_RESULT Comm_DevInit(void *pParams) 
{ 
    UINT8 *ptr = (UINT8*)pParams;

    UINT8 *portname = &ptr[0];
    UINT32 baudrate  = (ptr[10] << 16) | (ptr[11] << 8) | (ptr[12]);
    return InitComm(portname, baudrate);
}

/*
* Function : Comm_DevRead
* Purpose  : Device Read routine. The function finds the host port id and calls 
*            the device read routune from the ioDriverList array 
* Inputs   : UINT8 *readDataPtr pointer where read bytes will be stored
SINT32 requested , number of bytes to be read
SINT32 *actual , actual number of bytes read
void *pParams specifal flags for read operation e.g. BLOCK or NON BLOCK read calls
* Outputs  : None
Returns  :E_GENERIC__FAIL For failure
E_GENERIC__OK,  For success
*/

T_RESULT Comm_DevRead(UINT8 *readDataPtr, SINT32 requested, SINT32 *actual,void *pParams) 
{ 
    return ReadComm(readDataPtr, requested, actual);
}

/*
* Function : Comm_DevWrite
* Purpose  : Device Write routine. The function finds the host port id and calls 
*            the device write routune from the ioDriverList array
* Inputs   : UINT8 *writeDataPtr pointer to data to be written over device
SINT32 requested number of bytes to be sent 
SINT32 *actual actual number of bytes written
void *pParams special flags for writing operations e.g. BLOCK or NON BLOCK in case of UART
* Outputs  : None
Returns  :E_GENERIC__FAIL For failure
E_GENERIC__OK,  For success
*/

T_RESULT Comm_DevWrite(UINT8 *writeDataPtr,SINT32 requested, SINT32 *actual,void *pParams)
{
    return WriteComm(writeDataPtr, requested, actual);
}

/*
* Function : Comm_DevIOCtrl
* Purpose  : Device Io Control routine. The function finds the host port id and calls 
*            the device ioctl routune from the ioDriverList array
* Inputs   : SINT32 ioCtl value which indicates the io ctrl operations to be performed on device
void *pParams  pointer to parameters required for io ctrl operation
* Outputs  : None
Returns  :E_GENERIC__FAIL For failure
E_GENERIC__OK,  For success
*/

T_RESULT Comm_DevIOCtrl(SINT32 ioCtl, void *pParams)     
{ 
    if(ioCtl == UARTSetBaudrate)
    {
        return ChangeCommBaudRate(*(SINT32*)pParams);
    }
    else
    {
        DebugPrint(("UART IO CTL INVALID\n"));
        return     FAILURE;
    }

}

T_RESULT Comm_DevClose(void *pParams)
{
    CloseComm();
    return ERROR_NONE;
}



