/*                                                                        *
*    File is Copyright  2013 CSR Technology Inc.  All rights reserved.   *
*    No implied or expressed warranties are granted.                      *
*                                                                         *
*    This Software is protected by United States copyright laws and       *
*    international treaties.  You may not reverse engineer, decompile     *
*    or disassemble this Software.                                        *
*                                                                         *
*    WARNING:                                                             *
*    This Software contains CSR Technology Inc.s confidential and        *
*    proprietary information. UNAUTHORIZED COPYING, USE, DISTRIBUTION,    *
*    PUBLICATION, TRANSFER, SALE, RENTAL OR DISCLOSURE IS PROHIBITED      *
*    AND MAY RESULT IN SERIOUS LEGAL CONSEQUENCES.  Do not copy this      *
*    Software without CSR Technology, Inc.s  express written             *
*    permission. Use of any portion of the contents of this Software      *
*    is subject to and restricted by your signed written agreement with   *
*    CSR Technology, Inc.                                                 *
*                                                                         *
***************************************************************************
*
* MODULE:  ROM Patch Application
*
* FILENAME:  CommInterface.h
*
* DESCRIPTION:Declaration of functions related to serial communication .
*
***************************************************************************
*
*  Keywords for Perforce.  Do not modify.
*
*  $File: //core/tools/ROMPatchApplication/src/abstraction/CommInterface.h $
*
*  $DateTime: 2013/02/27 23:48:03 $
*
*  $Revision: #2 $
*
***************************************************************************/
#ifndef __COMMDRV_H__
#define __COMMDRV_H__

/*Enum describing commands  for io driver */
typedef enum _IoCtl
{
    UARTSetBaudrate
}tIoCtl;


/* exported functions  */

/**
* \brief  Dev Initialization routine. The function finds the host port id and calls 
*           the device initialization routune from the ioDriverList array
* \param pParams[IN] 
*            Pointer to initialization structure.
* \return E_GENERIC__FAIL For failure
E_GENERIC__OK,  For success
*/
extern T_RESULT Comm_DevInit(void *pParams);
/**
* \brief  Device Read routine. The function finds the host port id and calls 
*            the device read routune from the ioDriverList array 
* \param readDataPtr[IN]
pointer where read bytes will be stored
\param requested[IN] 
number of bytes to be read
\param actual[IN] 
actual number of bytes read
\param pParams[IN]
specifal flags for read operation e.g. BLOCK or NON BLOCK read calls
* 
* \return E_GENERIC__FAIL For failure
E_GENERIC__OK,  For success
*/


extern T_RESULT Comm_DevRead(UINT8 *readDataPtr, SINT32 requested, SINT32 *actual,void *pParams);
/**
* \brief  Device Write routine. The function finds the host port id and calls 
*            the device write routune from the ioDriverList array
* \param writeDataPtr[IN] 
pointer to data to be written over device
\param requested[IN] 
number of bytes to be sent 
\param actual[IN]
actual number of bytes written
\param pParams[IN] 
special flags for writing operations e.g. BLOCK or NON BLOCK in case of UART
* 
* \return E_GENERIC__FAIL For failure
E_GENERIC__OK,  For success
*/

extern T_RESULT Comm_DevWrite(UINT8 *writeDataPtr,SINT32 requested, SINT32 *actual,void *pParams);
/**
* \brief  Device Io Control routine. The function finds the host port id and calls 
*            the device ioctl routune from the ioDriverList array

* \param ioCtl[IN]
value which indicates the io ctrl operations to be performed on device
\param pParams[IN]  
pointer to parameters required for io ctrl operation
* 
* \return E_GENERIC__FAIL For failure
E_GENERIC__OK,  For success
*/

extern T_RESULT Comm_DevIOCtrl(SINT32 ioCtl, void *pParams);     

/**
* \brief  Device Close routine. The function closes the communication devices

\param pParams[IN]  
pointer to parameters required for close operation e.g. DEVICE NAME
* 
* \return E_GENERIC__FAIL For failure
E_GENERIC__OK,  For success
*/

extern T_RESULT Comm_DevClose(void *pParams); 

#endif  /*__COMMDRV_H__ */