/*                                                                        *
*    File is Copyright  2013 CSR Technology Inc.  All rights reserved.   *
*    No implied or expressed warranties are granted.                      *
*                                                                         *
*    This Software is protected by United States copyright laws and       *
*    international treaties.  You may not reverse engineer, decompile     *
*    or disassemble this Software.                                        *
*                                                                         *
*    WARNING:                                                             *
*    This Software contains CSR Technology Inc.s confidential and        *
*    proprietary information. UNAUTHORIZED COPYING, USE, DISTRIBUTION,    *
*    PUBLICATION, TRANSFER, SALE, RENTAL OR DISCLOSURE IS PROHIBITED      *
*    AND MAY RESULT IN SERIOUS LEGAL CONSEQUENCES.  Do not copy this      *
*    Software without CSR Technology, Inc.s  express written             *
*    permission. Use of any portion of the contents of this Software      *
*    is subject to and restricted by your signed written agreement with   *
*    CSR Technology, Inc.                                                 *
*                                                                         *
***************************************************************************
*
* MODULE:  Adaptation Layer
*
* FILENAME:  Logs.c
*
* DESCRIPTION:This file contains the source code for logging mechanism.For now it uses the
*                          file system to log the data.
*
***************************************************************************
*
*  Keywords for Perforce.  Do not modify.
*
*  $File: //core/tools/ROMPatchApplication/src/abstraction/Logs.c $
*
*  $DateTime: 2015/07/30 19:58:52 $
*
*  $Revision: #4 $
*
***************************************************************************/

/* This file is platform specific. */

#include <windows.h>
#include <stdio.h>

#include "basicTypes.h"
#include "string_abstraction.h"
#include "../errorCodes.h"
#include "logs.h"

#define LOG_BUFFER_SIZE 7000
#ifdef ENABLE_LOG
FILE *fp = NULL;
#endif

UINT32 ErrorCode;

/* Initialize and Open the Log File */
SINT32 LogInit(void *ptr)
{
#ifdef ENABLE_LOG
    LONG size;

    fp=NULL;
    /* This function is supposed to initialize the logging mechanism .*/
    /* Currently file operations are used for logging mechanism */
    if(NULL == (fp = fopen((UINT8*)ptr, "w+")))
    {   
        return FAILURE;
    }

    /* Get File Size */
    fseek(fp,0, SEEK_END);
    size = ftell(fp);

    /* Rewind pointer */
    fseek(fp, 0, SEEK_SET);
    return ERROR_NONE;
#else   
    return ERROR_NONE;
#endif
}

/* Print the String in the log File*/
SINT32 PrintLog(SINT8 *pData1, ...)
{
    /* Print the logs in the log file in proper format */
    /* Currently logs are saved in a log file */
#ifdef ENABLE_LOG
    va_list args;
    SINT32 cnt;
    SINT8 buf[LOG_BUFFER_SIZE];
    UINT16 length = 0;

    if(NULL == fp)
        return FAILURE;

    /* initialize the buffer before using it */
    memset (&buf[0], 0,(sizeof(UINT8)*(LOG_BUFFER_SIZE)));
    {
        va_start (args,pData1);
        /* vsnprintf is used to ensure no overrun of message buffer */
        cnt = vsnprintf (&buf[0], LOG_BUFFER_SIZE, pData1, args);
        if ( cnt<0 )
        {
            va_end (args);
            return FAILURE;
        }
        length = PM_strlen (&buf[0]);
        fwrite((void*)&buf[0],1,length,fp);
        va_end(args);
        return ERROR_NONE;
    }
#else
    return ERROR_NONE;
#endif
}

/* Close the Log File */
SINT32 CloseLog()
{
    /* This function is used to close the log mechanism */
    /* Close file */
#ifdef ENABLE_LOG
    if(NULL != fp)
    {
        fclose(fp);
        fp = NULL;
        return ERROR_NONE;
    }
    else
    {
        DebugPrint(("Log file pointer is not valid\n"));
        return FAILURE;
    }
#else
    return FAILURE;
#endif
}

void SetErrorCode(UINT32 code)
{
   ErrorCode = code;
}

UINT32 GetErrorCode(void)
{
   return ErrorCode;
}
