/*                                                                        *
*    File is Copyright  2013 CSR Technology Inc.  All rights reserved.   *
*    No implied or expressed warranties are granted.                      *
*                                                                         *
*    This Software is protected by United States copyright laws and       *
*    international treaties.  You may not reverse engineer, decompile     *
*    or disassemble this Software.                                        *
*                                                                         *
*    WARNING:                                                             *
*    This Software contains CSR Technology Inc.s confidential and        *
*    proprietary information. UNAUTHORIZED COPYING, USE, DISTRIBUTION,    *
*    PUBLICATION, TRANSFER, SALE, RENTAL OR DISCLOSURE IS PROHIBITED      *
*    AND MAY RESULT IN SERIOUS LEGAL CONSEQUENCES.  Do not copy this      *
*    Software without CSR Technology, Inc.s  express written             *
*    permission. Use of any portion of the contents of this Software      *
*    is subject to and restricted by your signed written agreement with   *
*    CSR Technology, Inc.                                                 *
*                                                                         *
***************************************************************************
*
* MODULE:  Adaptation Layer
*
* FILENAME:  fileOps.c
*
* DESCRIPTION: This file provides API's for accessing the file system. Internally this 
*                        accesses the file system.
*
***************************************************************************
*
*  Keywords for Perforce.  Do not modify.
*
*  $File: //core/tools/ROMPatchApplication/src/abstraction/fileOps.c $
*
*  $DateTime: 2013/02/27 23:48:03 $
*
*  $Revision: #3 $
*
***************************************************************************/


/* This file is platform specific. */
#include <windows.h>
#include <stdio.h>

#include "basicTypes.h"
#include "fileOps.h"
#include "../errorCodes.h"

/* Initialize and Open the File and set the file pointer at the start of the file */
/* Input File Name , Open Mode */
SINT32 PM_OpenFile(void *ptr)
{
    FILE *fp=NULL;

    fileOpenOpsStruct *fileOpenPtr = (fileOpenOpsStruct*)ptr;

    /* This function is supposed to initialize the logging mechanism .*/
    /* Currently file operations are used for logging mechanism */

    if(NULL == (fp = fopen((SINT8*)(fileOpenPtr->fileName),fileOpenPtr->mode)))
    {   
        return FAILURE;
    }

    fileOpenPtr->filePtr = (void*)fp;
    return ERROR_NONE;

}


/* Read the content of the file, 
given file identifier, offset from the starting of the file, number of bytes to read*/
SINT32 PM_ReadFile(void *opsStructPtr)
{
    UINT16 i;
    fileRWOpsStruct *localPtr = (fileRWOpsStruct*)opsStructPtr;

    /* Rewind pointer */
    fseek((FILE*)(localPtr->filePtr), 0, SEEK_SET);

    /* Set pointer */
    fseek((FILE*)(localPtr->filePtr), localPtr->offset1, SEEK_SET);

    /* In Ideal Case , when file has sufficient number of bytes */
    localPtr->offset2 = localPtr->offset1 + localPtr->numBytes;
    /* Load Data */
    for(i=0; i<localPtr->numBytes; i++)
        fread(&localPtr->ptr[i], 1, sizeof(UINT8), (FILE*)(localPtr->filePtr));

    return ERROR_NONE;
}

/* Write the bytes into the file 
Given File Identifier, offset from the starting of the file, number of bytes, data pointer */
SINT32 PM_WriteFile(void *opsStructPtr)
{
    fileRWOpsStruct *localPtr = (fileRWOpsStruct*)opsStructPtr;

    /* Rewind pointer */
    fseek((FILE*)(localPtr->filePtr), 0, SEEK_SET);

    /* Set pointer */
    fseek((FILE*)(localPtr->filePtr), localPtr->offset1, SEEK_SET);

    /* In Ideal Case , when file has sufficient number of bytes */
    localPtr->offset2 = localPtr->offset1 + localPtr->numBytes;

    fwrite((void*)(localPtr->ptr), 1, localPtr->numBytes, (FILE*)(localPtr->filePtr) );

    return ERROR_NONE;
}

/* Return the size of the file . If error return -1 */
SINT32 PM_GetFileSize(void *filePtr)
{
    LONG size;
    /* Get File Size */
    fseek((FILE*)filePtr,0, SEEK_END);
    size = ftell((FILE*)filePtr);
    return size;
}
/* Close the File */
/* Given File Identifier */
SINT32 PM_CloseFile(void *filePtr)
{
    fclose((FILE*)filePtr);
    return ERROR_NONE;
}
