/*                                                                        *
*    File is Copyright  2013 CSR Technology Inc.  All rights reserved.   *
*    No implied or expressed warranties are granted.                      *
*                                                                         *
*    This Software is protected by United States copyright laws and       *
*    international treaties.  You may not reverse engineer, decompile     *
*    or disassemble this Software.                                        *
*                                                                         *
*    WARNING:                                                             *
*    This Software contains CSR Technology Inc.s confidential and        *
*    proprietary information. UNAUTHORIZED COPYING, USE, DISTRIBUTION,    *
*    PUBLICATION, TRANSFER, SALE, RENTAL OR DISCLOSURE IS PROHIBITED      *
*    AND MAY RESULT IN SERIOUS LEGAL CONSEQUENCES.  Do not copy this      *
*    Software without CSR Technology, Inc.s  express written             *
*    permission. Use of any portion of the contents of this Software      *
*    is subject to and restricted by your signed written agreement with   *
*    CSR Technology, Inc.                                                 *
*                                                                         *
***************************************************************************
*
* MODULE:  Adaptation Layer
*
* FILENAME:  fileOps.h
*
* DESCRIPTION:Declaration of the API's for accessing the file system
*
***************************************************************************
*
*  Keywords for Perforce.  Do not modify.
*
*  $File: //core/tools/ROMPatchApplication/src/abstraction/fileOps.h $
*
*  $DateTime: 2013/02/27 23:48:03 $
*
*  $Revision: #2 $
*
***************************************************************************/



/** 
* \file   
* This file contains data type definitions
* 
*/
#ifndef _FILEOPS_H_
#define _FILEOPS_H_

typedef void* PM_FILEPTR;

typedef enum _fileOffsetMode
{
    FILE_OP_CURRENT=0, /* Read or Write at current offset */
    FILE_OP_START,     /* Read or Write from starting of the file */
    FILE_OP_OFFSET     /* Read or Write at the given offset */
}tfileOffsetMode;

typedef struct _fileOpenOps
{
    void *filePtr;  /* file pointer to be returned*/
    UINT8 fileName[100]; /* file name with complete path */
    UINT8 mode[5]; /* "rw","w+", etc */
}fileOpenOpsStruct;

typedef struct _fileRWOps
{
    void *filePtr;  /* file pointer */
    UINT32 offset1; /* offset at which read/write to perform */
    UINT32 offset2; /* offset after read/write */
    UINT32 numBytes; /* Number of bytes to read or write */
    UINT8 *ptr; /* read or write buf */
}fileRWOpsStruct;


/* Initialize and Open the File and set the file pointer at the start of the file */
/* Input File Name , Open Mode */
SINT32 PM_OpenFile(void *ptr);

/* Read the content of the file, 
given file identifier, offset from the starting of the file, number of bytes to read*/
SINT32 PM_ReadFile(void *opsStructPtr);

/* Write the bytes into the file 
Given File Identifier, offset from the starting of the file, number of bytes, data pointer */
SINT32 PM_WriteFile(void *opsStructPtr);

/* Close the File */
/* Given File Identifier */
SINT32 PM_CloseFile(void *filePtr);

/* Return the size of the file FilePtr is the input */
SINT32 PM_GetFileSize(void *filePtr);

#endif /*_FILEOPS_H_ */

