/*                                                                        *
*    File is Copyright  2013 CSR Technology Inc.  All rights reserved.   *
*    No implied or expressed warranties are granted.                      *
*                                                                         *
*    This Software is protected by United States copyright laws and       *
*    international treaties.  You may not reverse engineer, decompile     *
*    or disassemble this Software.                                        *
*                                                                         *
*    WARNING:                                                             *
*    This Software contains CSR Technology Inc.s confidential and        *
*    proprietary information. UNAUTHORIZED COPYING, USE, DISTRIBUTION,    *
*    PUBLICATION, TRANSFER, SALE, RENTAL OR DISCLOSURE IS PROHIBITED      *
*    AND MAY RESULT IN SERIOUS LEGAL CONSEQUENCES.  Do not copy this      *
*    Software without CSR Technology, Inc.s  express written             *
*    permission. Use of any portion of the contents of this Software      *
*    is subject to and restricted by your signed written agreement with   *
*    CSR Technology, Inc.                                                 *
*                                                                         *
***************************************************************************
*
* MODULE:  Driver
*
* FILENAME:  serialDriver.c
*
* DESCRIPTION:This file contains the source for serial driver which is used by the 
*                        communication API's to exchange data from receiver
*
***************************************************************************
*
*  Keywords for Perforce.  Do not modify.
*
*  $File: //core/tools/ROMPatchApplication/src/abstraction/serialDriver.c $
*
*  $DateTime: 2015/07/30 19:58:52 $
*
*  $Revision: #5 $
*
***************************************************************************/

#include <windows.h>
#include <string.h>
#include <stdio.h>
#include "basicTypes.h"
#include "logs.h"
#include "string_abstraction.h"
#include "../errorCodes.h"

HANDLE hComm;
DCB OldCommDCB, NewCommDCB;
COMMTIMEOUTS OldTimeouts, NewTimeouts;
DWORD OldCommMask, NewCommMask;
/*
* Function: InitComm
* Purpose:  Open communication channel
*/
#define MAX_CONFIG_SIZE _MAX_PATH

SINT32 InitComm(SINT8 *portname, UINT32 baudrate)
{
    SINT8 PortName[MAX_CONFIG_SIZE];

    PM_strcpy(PortName, "\\\\.\\");
    PM_strcat(PortName, portname);

    if ((hComm = CreateFile( PortName, GENERIC_READ | GENERIC_WRITE,
        0,                    /* exclusive access */
        NULL,                 /* no security attrs */
        OPEN_EXISTING,
        FILE_ATTRIBUTE_NORMAL,
        NULL )) == (HANDLE) -1 )
        return FAILURE;

    /* Get comm mask */

    GetCommMask(hComm, &OldCommMask);

    /* Set communication timeouts */

    GetCommTimeouts(hComm, &OldTimeouts);

    memcpy(&NewTimeouts, &OldTimeouts, sizeof(COMMTIMEOUTS));

    NewTimeouts.ReadIntervalTimeout = 10000;            /* 10 seconds between two chars max */
    NewTimeouts.ReadTotalTimeoutMultiplier = 10000;     /* or 5 seconds per char for whole message */
    NewTimeouts.ReadTotalTimeoutConstant = 10000;    /* plus 10 seconds on the top of that */

    SetCommTimeouts(hComm, &NewTimeouts);

    /* Set baud rate on selected comm port */

    GetCommState(hComm, &OldCommDCB);

    memcpy(&NewCommDCB, &OldCommDCB, sizeof(DCB));

    /* Basic elements : speed, no. of bits, no. of stop bits, parity */

    NewCommDCB.BaudRate = baudrate;
    NewCommDCB.ByteSize = 8;
    NewCommDCB.Parity = NOPARITY;
    NewCommDCB.StopBits = ONESTOPBIT;
    NewCommDCB.fBinary = 1;

    /* Flow control related stuff */

    NewCommDCB.fInX = FALSE;
    NewCommDCB.fOutX = FALSE;
    NewCommDCB.fOutxDsrFlow = FALSE;
    NewCommDCB.fOutxCtsFlow = FALSE;
    NewCommDCB.fDtrControl = DTR_CONTROL_DISABLE;
    NewCommDCB.fRtsControl = RTS_CONTROL_DISABLE;

    /* Activate new set of features */

    if (SetCommState(hComm, &NewCommDCB) == 0)
        return FAILURE;
    else
        return ERROR_NONE;
}

/*
* Function: ChangeCommBaudRate
* Purpose:  Change current baud rate on the port.
*/

SINT32 ChangeCommBaudRate(unsigned baudrate)
{
    if (NewCommDCB.BaudRate == baudrate)
        return ERROR_NONE;
    else
    {
        NewCommDCB.BaudRate = baudrate;

        /* Activate new set of features */
        if (SetCommState(hComm, &NewCommDCB) == 0)
        {
            DebugPrint(("Failed to change Patch Mgr Baud Rate %d\n",baudrate));
            return FAILURE;
        }
        else
        {
            return ERROR_NONE;
        }
    }
}


void SetONTimeOut()
{

    COMMTIMEOUTS OldTimeouts_1, NewTimeouts_1;
    /* Set communication timeouts */

    GetCommTimeouts(hComm, &OldTimeouts_1);

    memcpy(&NewTimeouts_1, &OldTimeouts_1, sizeof(COMMTIMEOUTS));

    NewTimeouts_1.ReadIntervalTimeout = 2000;            /* two seconds between two chars max */
    NewTimeouts_1.ReadTotalTimeoutMultiplier = 4;     /* or 4 milliseconds per char for whole message */
    NewTimeouts_1.ReadTotalTimeoutConstant = 2000;    /* plus 2 seconds on the top of that */

    SetCommTimeouts(hComm, &NewTimeouts_1);

}

void SetBackTimeOut()
{
    SetCommTimeouts(hComm, &NewTimeouts);
}
/*
* Function: ReadComm
* Purpose: Read specified number of characters from communication channel
*/

SINT32 ReadComm(UINT8 *p, LONG requested, LONG *actual)
{
    SINT32 n;
    n = ReadFile(hComm, p, requested, actual, NULL);

    return n;
}

/*
* Function: WriteComm
* Purpose: Write specified number of characters to communication channel
*/

SINT32 WriteComm(UINT8 *p, LONG requested, LONG *actual)
{
    return WriteFile(hComm, p, requested, actual, NULL);
}

/*
* Function: CloseComm
* Purpose: Close communication channel
*/

void CloseComm(void)
{
    /*DelayMs(1000); */
    SetCommTimeouts(hComm, &OldTimeouts);
    SetCommState(hComm, &OldCommDCB);
    SetCommMask(hComm, OldCommMask);
    CloseHandle(hComm);
}

