/*                                                                        *
*    File is Copyright  2013 CSR Technology Inc.  All rights reserved.   *
*    No implied or expressed warranties are granted.                      *
*                                                                         *
*    This Software is protected by United States copyright laws and       *
*    international treaties.  You may not reverse engineer, decompile     *
*    or disassemble this Software.                                        *
*                                                                         *
*    WARNING:                                                             *
*    This Software contains CSR Technology Inc.s confidential and        *
*    proprietary information. UNAUTHORIZED COPYING, USE, DISTRIBUTION,    *
*    PUBLICATION, TRANSFER, SALE, RENTAL OR DISCLOSURE IS PROHIBITED      *
*    AND MAY RESULT IN SERIOUS LEGAL CONSEQUENCES.  Do not copy this      *
*    Software without CSR Technology, Inc.s  express written             *
*    permission. Use of any portion of the contents of this Software      *
*    is subject to and restricted by your signed written agreement with   *
*    CSR Technology, Inc.                                                 *
*                                                                         *
***************************************************************************
*
* MODULE:  ROM Patch Application
*
* FILENAME:  Protocol.h
*
* DESCRIPTION:This file contains the serial protocol related information.
*
***************************************************************************
*
*  Keywords for Perforce.  Do not modify.
*
*  $File: //core/tools/ROMPatchApplication/src/protocol.h $
*
*  $DateTime: 2013/02/27 23:48:03 $
*
*  $Revision: #4 $
*
***************************************************************************/

#ifndef __PROTOCOL_H__
#define __PROTOCOL_H__


/* Modified these delays to allow program to work with our USB to Serial Driver */
#define DELAY_SWITCH_TO_SSB         100    /* Was 50, worked using 1000, 500 and 100 */
#define DELAY_AFTER_RESET           1000
#define DELAY_AFTER_BAUDRATECHANGE  100   /* Was 50, worked using 200 and 100 */

/*
* SPMessageId -- enumeration of message identifiers and error ids.
*
* !!!!!! when you add MID_'s also update sirfdemo code mid2str.cpp
*
* DATA:
*
* -- These are sirf serial protocol message data items.
*
* SP_TRANSPORT_STX_LEN --  the length of the transport header
* SP_TRANSPORT_ETX_LEN --  the length of the transport tailer
*
* SP_STX0        -- the first of the two start of tx characters
* SP_STX1        -- the second of the two start of tx characters
* SP_ETX0        -- the first of the two end of tx characters
* SP_ETX1        -- the second of the two end of tx characters
* SP_EXT_LEN_MASK-- mask that defines length that extends into 2nd byte
*/

/* SiRF protocol definitions */
enum _SiRFProtcolValue
{
    SP_TRANSPORT_STX_LEN = 2,
    SP_TRANSPORT_ETX_LEN = 2,
    SP_TRANSPORT_MSG_LEN = 2,
    SP_TRANSPORT_CKS_LEN = 2,
    SP_TRANSPORT_OVERHEAD = SP_TRANSPORT_STX_LEN + SP_TRANSPORT_ETX_LEN +
    SP_TRANSPORT_MSG_LEN + SP_TRANSPORT_CKS_LEN,
    SP_STX0 = 0xA0,
    SP_STX1 = 0xA2,
    SP_ETX0 = 0xB0,
    SP_ETX1 = 0xB3,

    /* Use this number and subtact a small constant to set max size */
    SP_MAX_MSG_LEN_EXPONET = 0x0F,         /* never allow sign bit to be set   */
    SP_MAX_MSG_LEN = (1<<(SP_MAX_MSG_LEN_EXPONET-4)) - 1, /* length; pwr of 2! */
    SP_EXT_LEN_WMASK = SP_MAX_MSG_LEN,
    SP_EXT_LEN_MASK = SP_EXT_LEN_WMASK>>8, /* high byte mask by shifting over 8*/
    SP_CKSUM_EXPONET = 0x0F,               /* never allow sign bit to be set   */
    Last_SiRFProtocolValue
};

#define SP_CKSUM_MASK  0x7FFF

typedef enum _SPMessgeId
{
    MID_Zero                     = 0x00,
    MID_LookInMessage            = MID_Zero,

    /* MODULE --> DEMO MESSAGES */

    MID_TrueNavigation           = 0x01,
    MID_MeasuredNavigation       = 0x02,
    MID_TrueTracker              = 0x03,
    MID_MeasuredTracker          = 0x04,
    MID_SWVersion                = 0x06,
    MID_ClockStatus              = 0x07,
    MID_50BPS                    = 0x08,
    MID_ThrPut                   = 0x09,
    MID_Error                    = 0x0A,
    MID_Ack                      = 0x0B,
    MID_Nak                      = 0x0C,
    MID_VisList                  = 0x0D,
    MID_Almanac                  = 0x0E,
    MID_Ephemeris                = 0x0F,
    MID_RawDGPS                  = 0x11,
    MID_OkToSend                 = 0x12,
    MID_RxMgrParams              = 0x13,

    MID_NetAssistReq             = 0x15,  /* E911 */
    MID_StopOutput               = 0x16,  /* E911 */
    MID_CompactTracker           = 0x17,  /* E911 */
    MID_DRCritSave               = 0x18,  /* SiRFDRive */
    MID_DRStatus                 = 0x19,  /* SiRFDRive */
    MID_DRHiRateNav              = 0x1A,  /* SiRFDRive */
    MID_DGPSStatus               = 0x1B,

    MID_NL_MeasData              = 0x1C,  /* nav lib; nav measurement info  */
    MID_NL_DGPSData              = 0x1D,  /* nav lib; DGPS measurement info */
    MID_NL_SVStateData           = 0x1E,  /* nav lib; SV state vector info  */
    MID_NL_InitData              = 0x1F,  /* nav lib; init info             */

    MID_TrkOnlyMeasData          = 0x20,
    MID_TrkOnlySV_Data           = 0x21,
    MID_TrkOnlySBAS_Data         = 0x22,
    MID_TrkOnlyRTC_Data          = 0x23,
    MID_TrkRollover              = 0x24,

    MID_TrkInit                  = 0x25,
    MID_TrkCommand               = 0x26,
    MID_TrkReset                 = 0x27,
    MID_TrkDownload              = 0x28,
    MID_GeodeticNav              = 0x29,
    MID_TrkPPS                   = 0x2A,
    MID_CMD_PARAM                = 0x2B,

    MID_TestModeData3            = 0x2E,  /* Test Mode 3 Data */
    MID_SiRFLocOut               = 0x2F,
    MID_SbasParam                = 0x32,
    MID_TestModeSS3TrackData     = 0x37,  /* Test Mode Meas */

    MID_Peek_Response            = 0x60,
    MID_UserOutputBegin          = 0x61,
    MID_EEGPSTimeInfo            = 0x38,
    MID_UserOutputEnd            = 0x7F,

    MID_NavigationInitialization = 0x80,
    MID_SetNMEAMode              = 0x81,
    MID_SetAlmanac               = 0x82,
    MID_FormattedDump            = 0x83,
    MID_PollSWVersion            = 0x84,
    MID_DGPSSourceControl        = 0x85,
    MID_SetSerialPort            = 0x86,
    MID_SetProtocol              = 0x87,
    MID_SET_NAV_MODE             = 0x88,
    MID_SET_DOP_MODE             = 0x89,
    MID_SET_DGPS_MODE            = 0x8A,
    MID_SET_ELEV_MASK            = 0x8B,
    MID_SET_POWER_MASK           = 0x8C,
    MID_SET_EDITING_RES          = 0x8D,
    MID_SET_SS_DETECTOR          = 0x8E,
    MID_SET_STAT_NAV             = 0x8F,
    MID_PollClockStatus          = 0x90,
    MID_SetDGPSPort              = 0x91,
    MID_PollAlmanac              = 0x92,
    MID_PollEphemeris            = 0x93,
    MID_FlashUpdate              = 0x94,
    MID_SetEphemeris             = 0x95,
    MID_SwitchOpMode             = 0x96,
    MID_LowPower                 = 0x97,
    MID_PollRxMgrParams          = 0x98,
    MID_TOWSync                  = 0x99,  /* E911 */
    MID_PollTOWSync              = 0x9A,  /* E911 */
    MID_EnableTOWSyncInterrupt   = 0x9B,  /* E911 */
    MID_TOWSyncPulseResult       = 0x9C,  /* E911 */
    MID_DRSetup                  = 0x9D,  /* SiRFDRive */
    MID_DRData                   = 0x9E,  /* SiRFDRive */
    MID_DRCritLoad               = 0x9F,  /* SiRFDRive */

    MID_HeadSync0xA0             = 0xA0,
    MID_HeadSync0xA2             = 0xA2,

    MID_E911HSPingRef            = 0xA3,  /* E911 */
    MID_E911RefPingHS            = 0xA4,  /* E911 */
    MID_ChangeUartChnl           = 0xA5,
    MID_SetMsgRate               = 0xA6,
    MID_LPAcqParams              = 0xA7,  /* new TricklePower parameters */
    MID_POLL_CMD_PARAM           = 0xA8,

    MID_SetSbasPrn               = 0xAA,  /* SBAS Parameters */
    MID_SiRFLocIn                = 0xAB,
    MID_AdvancedNavInit          = 0xAC,

    MID_SendCommandString        = 0xAF,  /* send debug string with parameters */

    MID_TailSync0xB0             = 0xB0,

    MID_SWToolBox                = 0xB2,  /* OSP Patch Protocol, Tracker Peek Poke */

    MID_TailSync0xB3             = 0xB3,

    MID_UserInputBegin           = 0xB4,
    MID_UserInputEnd             = 0xC7,
    MID_SetGenericSWControl      = 0xCD,

    MID_SiRFLoc                  = 0xD0,

    MID_Peek_Poke_Command        = 0xE0,

    MID_SiRFInternal             = 0xE4,
    MID_EEInputEphemeris         = 0xE8,
    MID_SetRFParams              = 0xE9, /* for RF Messages */
    MID_ASCIIData                = 0xFF,

    /* Error codes from here down! */
    MID_BufferFull = 257,
    MID_ParityError,
    MID_RcvFullError,
    MID_RcvOverrunError,
    MID_FrameError,
    MID_BreakInterrupt,
    MID_BufferTerminated,
    MID_TransportDataError,
    MID_CheckSumError,
    MID_LengthError,
    MID_MessageTypeError,
    Last_SPMessageId
} SPMessageId;

/* Sub-Ids for MID_SWToolBox_Input (0xB2)*/

/* Patch Message sent in to CSR GPS Chip */
#define SID_PM_START      0x28
#define SID_PM_LOAD       0x22
#define SID_PM_EXIT       0x26

/* Patch Message sent out by CSR GPS Chip */
#define SID_PM_PROMPT     0x90
#define SID_PM_ACK        0x91

/* Nav/Tracker Peek Poke SIDs */
#define SID_PEEKPOKE_CMD  0x03
#define SID_PEEKPOKE_RSP  0x04

# ifndef FIRST_VALID_MSG_ID 
# define FIRST_VALID_MSG_ID (0)
# endif
# ifndef LAST_VALID_MSG_ID 
# define LAST_VALID_MSG_ID (255)
# endif


typedef enum _rxEventType
{
    response_PeekPoke            = 0x4,
    response_PollNavParams       = 0x13,
    response_PollSwVersion       = 0xf,
    response_SetSerialBitRate    = 0x86,
    response_ChangeProtocol      = 0x87,
    response_PMPrompt            = 0x90,      /* OSP Patch Protocol */
    response_PMAck               = 0x91,
    response_recvSSB             = 0xff
}rxEventType;

typedef enum MessageId
{
    First_ValidMessageId = FIRST_VALID_MSG_ID,
    Last_ValidMessageId = LAST_VALID_MSG_ID,
    ValidMessageRange = Last_ValidMessageId - First_ValidMessageId + 1
} MessageId;

#endif  /* __PROTOCOL_H__ */
