/*                                                                        *
*    File is Copyright  2013 CSR Technology Inc.  All rights reserved.   *
*    No implied or expressed warranties are granted.                      *
*                                                                         *
*    This Software is protected by United States copyright laws and       *
*    international treaties.  You may not reverse engineer, decompile     *
*    or disassemble this Software.                                        *
*                                                                         *
*    WARNING:                                                             *
*    This Software contains CSR Technology Inc.s confidential and        *
*    proprietary information. UNAUTHORIZED COPYING, USE, DISTRIBUTION,    *
*    PUBLICATION, TRANSFER, SALE, RENTAL OR DISCLOSURE IS PROHIBITED      *
*    AND MAY RESULT IN SERIOUS LEGAL CONSEQUENCES.  Do not copy this      *
*    Software without CSR Technology, Inc.s  express written             *
*    permission. Use of any portion of the contents of this Software      *
*    is subject to and restricted by your signed written agreement with   *
*    CSR Technology, Inc.                                                 *
*                                                                         *
***************************************************************************
*
* MODULE:  ROM Patch Application
*
* FILENAME:  ssbMgr.c
*
* DESCRIPTION:This file contains the source for serial protocol and handlers of the SSB 
*                        message
*
***************************************************************************
*
*  Keywords for Perforce.  Do not modify.
*
*  $File: //core/tools/ROMPatchApplication/src/ssbMgr.c $
*
*  $DateTime: 2015/11/12 20:51:56 $
*
*  $Revision: #13 $
*
***************************************************************************/

/**************************************************************************/
/*                              INCLUDES                                  */
/**************************************************************************/
#include "Abstraction/basicTypes.h"
#include "Abstraction/osAbstraction.h"
#include "Abstraction/CommInterface.h"
#include "protocol.h"
#include "Abstraction/logs.h"
#include "crc.h"
#include "ROMPatchSettings.h"
#include "Abstraction/string_abstraction.h"
#include "errorCodes.h"
#include "ssbInterface.h"
#include "winbase.h"

/**************************************************************************/
/*                          GLOBALS                                       */
/**************************************************************************/
#define SSB_MAX_MSG_LEN                  1024

extern tInputConfig inputConfiguration;
extern tPMPatchState patchState;

UINT8 ResponseRcvd = 0;
UINT8 HelloOutput[SP_MAX_MSG_LEN];
UINT8 SwVersion[80];

/* states for the transport layer parsing state machine */
typedef enum
{
    START1,
    START2,
    LENGTH,
    PAYLOAD,
    CHECKSUM,
    END1,
    END2
} MessageStates;

static struct
{
    UINT8 buf[SSB_MAX_MSG_LEN + SP_TRANSPORT_CKS_LEN];
    UINT16 currentBufferOffset;
    UINT16 msgStartOffset;
    UINT16 mlen;
    UINT16 tempByteCount;
    UINT16 rcvdLen;
    UINT16 compChecksum;
    UINT16 rcvdChecksum;
    MessageStates SSBMessageState;
} SSB_SPMsg;                    /* message Data Structure */

/**************************************************************************/
/*              STATIC FUNCTION PROTOTYPES                                */
/**************************************************************************/
static SINT32 HandlePMPrompt(UINT8 *pMsg, SINT32 len);
static SINT32 HandlePMAck(UINT8 *pMsg, SINT32 len);
static void Print_PMExit_ErrorCode(UINT8 ErrorCode);
static SINT32 HandlePeekPokeResponse(UINT8 *pMsg, SINT32 len);
static SINT32 SSBmsgDeliver (MessageId mid, UINT8 * pMsg, SINT32 len);
static SINT32 startSM2 (UINT8 * pBuf, UINT16 msgLen, UINT16 bufferOffset);
static SINT32 spSSBDeliver (UINT8 * pBuf, SINT32 bufLen);
static SINT32 ComputeChksum( UINT8* pBuf, SINT32 Cnt );
static unsigned short NTOS( unsigned short n );
static unsigned short STON( unsigned short n );
static unsigned long NTOL( unsigned long n );
static unsigned long LTON( unsigned long n );
static void ImportUINT8(UINT8 *Dst, UINT8** pSrc);
static void ImportUINT16( WORD *Dst, UINT8** pSrc );
static void ImportINT16( short *Dst, UINT8** pSrc );
static void ImportUINT32( DWORD *Dst, UINT8** pSrc );
static void ImportINT32( long *Dst, UINT8** pSrc );
static void RcvrParamSS2Import( UINT8* pSrc );
static void PollNavParamsResponseHdlr(UINT8 * pMsg, SINT32 len);
static SINT32 PollSwVersionHdlr(UINT8* pMsg);

/**************************************************************************/
/*                     GLOBAL FUNCTIONS                                   */
/**************************************************************************/
SINT32 SSB_Send (void * msgPtr, UINT32 msgLen)
{
    UINT16 ckSum;
    UINT8 *pBuf=NULL;
    LONG actual;

#if 0
    { /* Debug section, makes it easy to see exactly what goes out... */
      static int C = 0;
      char str[64];
      unsigned char *p = (unsigned char *)msgPtr;
      int i;
      FILE *fil;
      C++;
      sprintf(str,"sent_msg_%d.txt",C);
      fil = fopen(str,"wb");
      for(i=0;i<msgLen;i++)
      {
         fprintf(fil,"%X%X ",p[i]>>4,p[i]&0xf);
         if( (i&31) == 31)
            fprintf(fil,"\r\n");
      }         
      fclose(fil);
    }
#endif

    if (msgPtr == NULL)
    {
        return (FAILURE);
    }

    pBuf = (UINT8*)PM_MemAlloc(msgLen+8);

    pBuf[0] = SP_STX0;
    pBuf[1] = SP_STX1;
    pBuf[2] = (UINT8) ((msgLen >> 8) & SP_EXT_LEN_MASK);
    pBuf[3] = (UINT8) msgLen;

    /*Payload */
    PM_MemCpy(pBuf+4,msgPtr,msgLen);

    ckSum = ComputeChksum (msgPtr,msgLen);
    ckSum &= SP_CKSUM_MASK;

    pBuf[msgLen+4] = (UINT8) (ckSum >> 8);
    pBuf[msgLen+5] = (UINT8) ckSum;
    pBuf[msgLen+6] = SP_ETX0;
    pBuf[msgLen+7] = SP_ETX1;

    Comm_DevWrite(pBuf, msgLen+8, &actual, NULL);

    PM_MemFree(pBuf);

    return ERROR_NONE;
}

SINT32 ListenRx(rxEventType event)
{
    UINT8 c;
    LONG actual;
    UINT8 dataStart = 0;
    UINT16 index =0;
    UINT8 numMsgRcvd = 0;
    SINT32 returnStatus = ERROR_NONE;
    UINT16 countRcvd = 0;

    PM_MemSet(HelloOutput, 0, SP_MAX_MSG_LEN);

#if 0
    /* Used for debugging */
    if (event == response_recvSSB)
       DebugPrint(("Waiting for SSB Event.\n"));
#endif

    while(1)
    {
        actual = 0;
        Comm_DevRead(&c, 1 , &actual,NULL);
        countRcvd++;
        if(actual != 0) 
        {
            if (c == SP_STX0)
            {
                dataStart = 1;
            }
        }
        if( (dataStart == 0) && (countRcvd >= SSB_MAX_MSG_LEN))
        {
            return ERROR_MAX_NUMMSG_WAIT_EXPIRED;
        }
        if(0 == actual )
        {
            returnStatus = ERROR_RCVR_NOT_RESPONDING;
            return returnStatus;
        }

        if(dataStart == 1)
        {
            if(index == SP_MAX_MSG_LEN)
            {
                index = 0;
                dataStart = 0;
                /* DebugPrint(("No End of msg within max byte\n")); */
            }
            else
            {
                HelloOutput[index++] = c;

                if((HelloOutput[index-1] == SP_ETX1) && (HelloOutput[index-2] == SP_ETX0))
                {  
                    switch(event)
                    {
                    case response_PollSwVersion:
                       if(HelloOutput[4] == MID_SWVersion)
                       {
                          ResponseRcvd = 1;
                       }
                       break;
                    case response_PollNavParams:
                       if( (HelloOutput[4] == MID_RxMgrParams) || 
                          (  (HelloOutput[4]  == MID_Nak) && (HelloOutput[5] == MID_PollRxMgrParams)) )
                       {
                          ResponseRcvd = 1;
                       }
                       break;
                    case response_PMPrompt:
                        if((HelloOutput[4] == MID_SWToolBox)&&(HelloOutput[5] == SID_PM_PROMPT))
                        {
                           ResponseRcvd = 1;
                        }
                        break;
                    case response_PMAck:
                        if((HelloOutput[4] == MID_SWToolBox) && (HelloOutput[5] == SID_PM_ACK))
                        {
                           ResponseRcvd = 1;
                        }
                        break;
                    case response_PeekPoke:
                       if((HelloOutput[4] == MID_SWToolBox)&&(HelloOutput[5] == SID_PEEKPOKE_RSP))
                       {
                          ResponseRcvd = 1;
                       }
                       break;
                    default:
                        break;
                    }

                    returnStatus = spSSBDeliver (&HelloOutput[0],index);
                    if(returnStatus == ERROR_NONE_SSB_MSG_RCVD)
                    {
                        numMsgRcvd++;
                        if(response_recvSSB == event)
                        {
                            ResponseRcvd =1 ;
                        }
                    }
                    index = 0;
                    dataStart = 0;
                    if(ResponseRcvd == 1)
                    {
                        ResponseRcvd = 0;
                        return returnStatus;
                    }
                    /* Wait for a few OSP messages before returning from this function */
                    if(numMsgRcvd > 50)   /* May need to tune this value?  Was 30 */
                    {
                        ResponseRcvd = 0;
                        return ERROR_MAX_NUMMSG_WAIT_EXPIRED;
                    }
                }
            }
        }
    }
}

SINT32 VerifyCommunication()
{
    SINT32 returnStatus = ListenRx(response_recvSSB);

    if(ERROR_NONE_SSB_MSG_RCVD == returnStatus)
    {
        DebugPrint(("\nCommunication with Rx verified (SUCCESS)\n"));
        return ERROR_NONE;
    }
    else if(ERROR_MAX_NUMMSG_WAIT_EXPIRED == returnStatus)
    {
        DebugPrint(("FAILURE! Rx could be in different baud rate and protocol..\n"));
        return ERROR_COM_BYTES_UNRECOGNIZED;
    }
    else  if(returnStatus == ERROR_RCVR_NOT_RESPONDING)
    {
        DebugPrint(("FAILURE! Verify Communication. \n"));
    }

    return returnStatus;
}

SINT32 PollSWVersion()
{
    UINT8 pData[2];
    SINT32 returnStatus = ERROR_NONE;

    pData[0] = MID_PollSWVersion;

    pData[1] = 0;

    SSB_Send(pData, 2);

    returnStatus = ListenRx(response_PollSwVersion);

    return returnStatus;
}
SINT32 PollNAVParameters()
{
    UINT8 pData[2];
    pData[0] = MID_PollRxMgrParams;
    pData[1] = 0x00; 
    SSB_Send(pData, 2);
    return ERROR_NONE;
}

SINT32 EnterFullPower()
{
    UINT8 pData[9];
    pData[0] = MID_LowPower;

    pData[1] = 0;
    pData[2] = 0;

    pData[3] = 0x03;
    pData[4] = 0xE8;

    pData[5] = 0x0;
    pData[6] = 0x0;
    pData[7] = 0x0;
    pData[8] = 0xC8;

    SSB_Send(pData, 9);

    return ERROR_NONE;

}

SINT32 EnterLowPower(UINT8 PowrMode)
{
    UINT8 pData[9];
    char  LP_Str[300] = "";
    char  tmpStr[100] = "";
    float LP_DutyCycle;

    if( (!SavedRcvrParamSS2.LP_OnTime) || (!SavedRcvrParamSS2.LP_Interval) )
    {
        LP_DutyCycle = 0.0;
    }
    else
    {
        LP_DutyCycle = (float) (100 * SavedRcvrParamSS2.LP_OnTime)
            / (float) (SavedRcvrParamSS2.LP_Interval);
    }

    pData[0] = MID_LowPower;
    if(2 == PowrMode)
    {
        /* if PTF then set the field */
        pData[1] = 0;
        pData[2] = 1;
    }
    else
    {
        pData[1] = 0;
        pData[2] = 0;
    }

    LP_DutyCycle *= 10;
    pData[3] = ((UINT32)LP_DutyCycle & 0xff00)>> 8;
    pData[4] = (UINT32)LP_DutyCycle & 0xff;
    pData[5] = (UINT8)((SavedRcvrParamSS2.LP_OnTime & 0xff000000) >> 24);
    pData[6] = (UINT8)((SavedRcvrParamSS2.LP_OnTime & 0xff0000) >> 16);
    pData[7] = (UINT8)((SavedRcvrParamSS2.LP_OnTime & 0xff00) >> 8);
    pData[8] = (UINT8)(SavedRcvrParamSS2.LP_OnTime & 0xff);

    SSB_Send(pData, 9);
    DebugPrint(("Rx LOW POWR Mode Requested\n"));

    return ERROR_NONE;
}

/* Note BitRate can not be > 65535 (MAX UINT16 value), so */
/* 115200 and higher bit rates can not be used for NMEA.  */
SINT32 SwitchSsbToNmea(UINT16 BitRate)
{
    UINT8 pData[24];
    pData[0] =  MID_SetNMEAMode;
    pData[1] =  0x02; /* Mode */
    pData[2] =  0x01; /* GGA message */
    pData[3] =  0x01; /* checksum */
    pData[4] =  0x00; /* GLL message */
    pData[5] =  0x01; /* checksum */
    pData[6] =  0x01; /* GSA message */
    pData[7] =  0x01; /* checksum */
    pData[8] =  0x05; /* GSV message */
    pData[9] =  0x01; /* checksum */
    pData[10] = 0x01; /* RMC message */
    pData[11] = 0x01; /* checksum */
    pData[12] = 0x00; /* VTG message */
    pData[13] = 0x01; /* checksum */
    pData[14] = 0x00; /* MSS message */
    pData[15] = 0x01; /* checksum */
    pData[16] = 0x00; /* EPE message */
    pData[17] = 0x00; /* checksum */
    pData[18] = 0x00; /* ZDA message */
    pData[19] = 0x01; /* checksum */
    pData[20] = 0x00; /* Unused field */
    pData[21] = 0x00; /* Unused field */
    pData[22] = (UINT8)((BitRate >> 8) & 0xFF);  
    pData[23] = (UINT8)(BitRate & 0xFF);

    /* Try sending the message few times */
    SSB_Send(pData, 24);
    DelayMs(100);
    SSB_Send(pData, 24);
    DelayMs(100);
    SSB_Send(pData, 24);

    return ERROR_NONE;
}

/* Found that sending MID166 to turn off OSP debug messages helps the */
/* ROMPatchApplication program process OSP response msgs like the Ack */
/* msg after sending the Poll Nav Parms command. Reducing the OSP msg */
/* traffic for messages not being interrogated reduces the likelihood */
/* of a patch load failure. Turning other messages like Nav Lib msgs  */
/* and MID64 will also help reduce the OSP msg traffic.               */

/* Example of MID166 that turns off OSP debug messages. */
/* A0 A2 00 08 A6 00 FF 00 00 00 00 00 01 A5 B0 B3 */
SINT32 SwitchNmeaToSsb(UINT32 baudrate)
{
    LONG actual;
    UINT8 index=0;
    UINT8 ckSum = 0;
    SINT8* pBuf;
    SINT8 *pData =  (SINT8*)PM_MemAlloc(40);
    T_RESULT writeRtn = 0;

    PM_MemSet(pData, 0, 40);
    PM_strcpy(pData, "\r\n$PSRF100,0,");
    index = PM_strlen("\r\n$PSRF100,0,");

    switch(baudrate)
    {
    case 4800:
        PM_strcpy(pData+index,"4800,8,1,0");
        index += PM_strlen("4800,8,1,0");
        break;
    case 9600:
        PM_strcpy(pData+index,"9600,8,1,0");
        index += PM_strlen("9600,8,1,0");
        break;
    case 19200:
        PM_strcpy(pData+index,"19200,8,1,0");
        index += PM_strlen("19200,8,1,0");
        break;
    case 38400:
        PM_strcpy(pData+index,"38400,8,1,0");
        index += PM_strlen("38400,8,1,0");
        break;
    case 57600:
        PM_strcpy(pData+index,"57600,8,1,0");
        index += PM_strlen("57600,8,1,0");
        break;
    case 115200:
        PM_strcpy(pData+index,"115200,8,1,0");
        index += PM_strlen("115200,8,1,0");
        break;
    case 230400:
        PM_strcpy(pData+index,"230400,8,1,0");
        index += PM_strlen("230400,8,1,0");
        break;
    case 460800:
        PM_strcpy(pData+index,"460800,8,1,0");
        index += PM_strlen("460800,8,1,0");
        break;
    default:
        DebugPrint(("Error %d: Switch NMEA to OSP Baud Rate %d Not Supported\n", ERROR_INVALID_BAUD, baudrate));
        SetErrorCode(ERROR_INVALID_BAUD);
        return FAILURE;
    }

    pBuf = pData + 3;

    while (*pBuf != '\0')
    {
        ckSum ^= *pBuf++;
    }

    sprintf(pData + index,"*%02X",ckSum); 

    PM_strcat(pData, "\r\n");
    /* Found that when using USB as a comm port the Comm_DevWrite function */
    /* would return 0 instead of 1. To fix this delays were increased.     */
    writeRtn = Comm_DevWrite(pData,index+5,&actual, NULL);
    DelayMs(DELAY_SWITCH_TO_SSB);
    Comm_DevIOCtrl(UARTSetBaudrate,(void*)&baudrate);
    PM_MemFree(pData);
    /* DelayMs(DELAY_SWITCH_TO_SSB); */

    return ERROR_NONE;
}

SINT32 SwitchNmeaBaud(UINT32 baudrate)
{
    LONG actual;
    UINT8 index=0;
    UINT8 ckSum = 0;
    SINT8* pBuf;
    SINT8 *pData =  (SINT8*)PM_MemAlloc(40);
    T_RESULT writeRtn = 0;

    PM_MemSet(pData, 0, 40);
    PM_strcpy(pData, "\r\n$PSRF100,1,");
    index = PM_strlen("\r\n$PSRF100,1,");

    switch(baudrate)
    {
    case 4800:
        PM_strcpy(pData+index,"4800,8,1,0");
        index += PM_strlen("4800,8,1,0");
        break;
    case 9600:
        PM_strcpy(pData+index,"9600,8,1,0");
        index += PM_strlen("9600,8,1,0");
        break;
    case 19200:
        PM_strcpy(pData+index,"19200,8,1,0");
        index += PM_strlen("19200,8,1,0");
        break;
    case 38400:
        PM_strcpy(pData+index,"38400,8,1,0");
        index += PM_strlen("38400,8,1,0");
        break;
    case 57600:
        PM_strcpy(pData+index,"57600,8,1,0");
        index += PM_strlen("57600,8,1,0");
        break;
    case 115200:
        PM_strcpy(pData+index,"115200,8,1,0");
        index += PM_strlen("115200,8,1,0");
        break;
    case 230400:
        PM_strcpy(pData+index,"230400,8,1,0");
        index += PM_strlen("230400,8,1,0");
        break;
    case 460800:
        PM_strcpy(pData+index,"460800,8,1,0");
        index += PM_strlen("460800,8,1,0");
        break;
    default:
        DebugPrint(("Error %d: Switch NMEA Baud Rate %d Not Supported\n",ERROR_INVALID_BAUD, baudrate));
        SetErrorCode(ERROR_INVALID_BAUD);
        return FAILURE;
    }

    pBuf = pData + 3;

    while (*pBuf != '\0')
    {
        ckSum ^= *pBuf++;
    }

    sprintf(pData + index,"*%02X",ckSum); 

    PM_strcat(pData, "\r\n");
    /* Found that when using USB as a comm port the Comm_DevWrite function */
    /* would return 0 instead of 1. To fix this delays were increased.     */
    writeRtn = Comm_DevWrite(pData,index+5,&actual, NULL);
    DelayMs(DELAY_AFTER_BAUDRATECHANGE);
    Comm_DevIOCtrl(UARTSetBaudrate,(void*)&baudrate);
    PM_MemFree(pData);

    return ERROR_NONE;
}


SINT32 SetSerialBitRate(UINT32 BitRate)
{
    UINT8 pData[9];

    pData[0] = MID_SetSerialPort;

    pData[1] = (BitRate >> 24) & 0xff;
    pData[2] = (BitRate >> 16) & 0xff;
    pData[3] = (BitRate >> 8) & 0xff;
    pData[4] = (BitRate) & 0xff;

    pData[5] = 8;
    pData[6] = 1;
    pData[7] = 0;
    pData[8] = 0;

    SSB_Send(pData, 9);
    SSB_Send(pData, 9);
    DelayMs(100);

    Comm_DevIOCtrl(UARTSetBaudrate,(void*)&BitRate);

    return ERROR_NONE;
}

/* Supports 4-byte peek/poke of Nav or Tracker address space */
SINT32 SendPeekPokeRequest(void *msg)
{
   UINT8 pData[12];

   tSIRF_MSG_SSB_TRKR_PEEKPOKE_CMD* p_msg;
   p_msg = (tSIRF_MSG_SSB_TRKR_PEEKPOKE_CMD*)msg;

   pData[0] = MID_SWToolBox;
   pData[1] = SID_PEEKPOKE_CMD;

   pData[2] =  p_msg->type;
   pData[3] =  p_msg->access;
   pData[4] =  (p_msg->address >> 24) & 0xff;
   pData[5] =  (p_msg->address >> 16) & 0xff;
   pData[6] =  (p_msg->address >> 8) & 0xff;
   pData[7] =  (p_msg->address) & 0xff;
   pData[8] =  (p_msg->data >> 24) & 0xff;
   pData[9] =  (p_msg->data >> 16) & 0xff;
   pData[10] = (p_msg->data >> 8) & 0xff;
   pData[11] = (p_msg->data) & 0xff;

   SSB_Send(pData, 12);

   return ERROR_NONE;
}

/**************************************************************************/
/*                     STATIC FUNCTIONS                                   */
/**************************************************************************/
static SINT32 HandlePMPrompt(UINT8 *pMsg, SINT32 len)
{
   SINT32 retval = FAILURE;
   tSIRF_MSG_SSB_PM_PROMPT_OUT prompt;
   tPMPatchFileInfo *pfi = &patchState.fileinfo;

   /* Check if message length is correct, just to be safe */
   if (len == sizeof(prompt))
   {
      prompt.ChipId = pMsg[0]<<8 | pMsg[1];
      prompt.SiliconVersion = pMsg[2]<<8 | pMsg[3];
      prompt.ROMVersionCode = pMsg[4]<<8 | pMsg[5];
      prompt.PatchVersionCode = pMsg[6]<<8 | pMsg[7];
      /* Copy pm prompt msg into global patch structure */
      memcpy(&patchState.pm_prompt,&prompt,sizeof(prompt));
      /* Check if ROM is compatible with patch file */
      if (((pfi->ChipId == prompt.ChipId) &&
           (pfi->SiliconVersion == prompt.SiliconVersion) &&
           (pfi->ROMVersionCode == prompt.ROMVersionCode)) &&
           (pfi->PatchRevisionCode >= prompt.PatchVersionCode))  /* Allows patch file with same version code to be reloaded. */
      {
           /* This flag indicates that patch loading of patch file should commence. */
           patchState.ValidLoadInProgress = TRUE;
           retval = ERROR_NONE;
      }
      else
      {
           DebugPrint(("Error %d: PM prompt msg not compatible with Rx\n", ERROR_PATCH_PM_PROMPT_MSG));
           SetErrorCode(ERROR_PATCH_PM_PROMPT_MSG);
           patchState.ValidLoadInProgress = FALSE;
      }
      
   }
   else
   {
      /* This should not happen */
      DebugPrint(("Error %d: Incorrect msg length for PM prompt msg.\n", ERROR_PATCH_PM_PROMPT_MSG));
      SetErrorCode(ERROR_PATCH_PM_PROMPT_MSG);
   }
   return(retval);
}

static SINT32 HandlePMAck(UINT8 *pMsg, SINT32 len)
{
   SINT32 retval = FAILURE;
   tSIRF_MSG_SSB_PM_ACK_OUT ack;
   tPMPatchFileState *pfs = &patchState.filestate;

   /* Check if message length is correct, just to be safe */
   if (len == sizeof(ack))
   {
      ack.MessageSeqNo = pMsg[0]<<8 | pMsg[1];
      ack.MessageIdAck = pMsg[2];
      ack.AckStatus = pMsg[3];

      switch(ack.MessageIdAck)
      {
        case SID_PM_LOAD:
           if ((ack.MessageSeqNo&0x7FFF) == pfs->seq) /* remove MSB for K32 support (have MSB set) */
           {
              /* 4e patch code will only respond with PM_MSG_RX_OK_OP_SUCCESS */
              /* or PM_MSG_RX_OK_OP_FAILURE for PM Load messages.             */
              if (ack.AckStatus == PM_MSG_RX_OK_OP_SUCCESS)
              {
                 pfs->ack_rcvd = TRUE;
                 retval = ERROR_NONE;
              }
              else
              {
                 /* Currently failure codes are not returned from PM Load ack msgs. */
                 DebugPrint(("Error %d: PM Load ack msg indicates Failure for Load Seq 0x%02x\n",ERROR_PATCH_PM_LOAD_MSG, ack.MessageSeqNo));
                 SetErrorCode(ERROR_PATCH_PM_LOAD_MSG);
              }
           }
           else
           {
              /* This should not happen. */
              DebugPrint(("Error %d: PM Load ack msg rcvd out of seq expect 0x%02x and rcvd 0x%02x\n",ERROR_PATCH_PM_LOAD_MSG, pfs->seq,ack.MessageSeqNo));
              SetErrorCode(ERROR_PATCH_PM_LOAD_MSG);
           }
           break;
        case SID_PM_EXIT:
           if (ack.MessageSeqNo == 0)
           {
              if (ack.AckStatus == PM_MSG_RX_OK_OP_SUCCESS)
              {
                 pfs->ack_rcvd = TRUE;
                 retval = ERROR_NONE;
                 DebugPrint(("Successful PM Exit ack received.\n"));
              }
              else
              {
                 if ((ack.AckStatus & PM_MSG_RX_OP_MASK) == PM_MSG_RX_OK_OP_FAILURE)
                 {
                    UINT8 OpFailureCode = 0;

                    OpFailureCode = (ack.AckStatus & PM_MSG_OP_FAIL_MASK)>>2;
                    DebugPrint(("Error %d: PM Exit ack indicates Operation Failure with Error Code 0x%02x\n",ERROR_PATCH_PM_EXIT_MSG, OpFailureCode));
                    /* Avoid setting error code for device access error because a customer may or may not have an external storage device */
                    if((ack.AckStatus & PM_MSG_OP_FAIL_MASK) == PATCH_RAM_CRC_FAILURE ||  
                       (ack.AckStatus & PM_MSG_OP_FAIL_MASK) == INCOMPLETE_PATCH
                       )
                    {
                       SetErrorCode(ERROR_PATCH_PM_EXIT_MSG);
                    }
                    Print_PMExit_ErrorCode(OpFailureCode);
                 }
                 else
                 {
                    /* This should not happen. */
                    DebugPrint(("Error %d: Invalid PM Exit ack received with unknown status 0x%02x\n",ERROR_PATCH_PM_EXIT_MSG, ack.AckStatus));
                    SetErrorCode(ERROR_PATCH_PM_EXIT_MSG);
                 }
              }
           }
           else
           {
              /* This should not happen. */
              DebugPrint(("Error %d: Invalid PM Exit ack received with non-zero msg sequence number\n", ERROR_PATCH_PM_EXIT_MSG));
              SetErrorCode(ERROR_PATCH_PM_EXIT_MSG);
           }
           break;
        default:
           {
              /* This should not happen */
              DebugPrint(("Error %d: PM ack rcvd for unknown mid 0x%02x\n",ERROR_ACK_RCVD, ack.MessageIdAck));
              SetErrorCode(ERROR_ACK_RCVD);
           }
           break;
      } /* End of switch(ack.MessageIdAck) */
   }
   else
   {
      /* This should not happen */
      DebugPrint(("Error %d: Incorrect msg length for PM ack msg.\n", ERROR_ACK_RCVD));
      SetErrorCode(ERROR_ACK_RCVD);
   }
   return(retval);
}

static void Print_PMExit_ErrorCode(UINT8 ErrorCode)
{
   DebugPrint(("OSP: Error Code is a bit mapped value.\n"));
   DebugPrint(("OSP: Error(s) indicate:\n"));
   if (ErrorCode & 0x01)
   {
      DebugPrint(("OSP: 0x01 - PATCH_RAM_CRC_FAILURE\n"));
   }
   if (ErrorCode & 0x02)
   {
      DebugPrint(("OSP: 0x02 - DEVICE_OPEN_ERROR\n"));
   }
   if (ErrorCode & 0x04)
   {
      DebugPrint(("OSP: 0x04 - DEVICE_CLOSE_ERROR\n"));
   }
   if (ErrorCode & 0x08)
   {
      DebugPrint(("OSP: 0x08 - DEVICE_WRITE_ERROR\n"));
   }
   if (ErrorCode & 0x10)
   {
      DebugPrint(("OSP: 0x10 - DEVICE_READ_ERROR\n"));
   }
   if (ErrorCode & 0x20)
   {
      DebugPrint(("OSP: 0x20 - INCOMPLETE_PATCH\n"));
   }
}

static SINT32 HandlePeekPokeResponse(UINT8 *pMsg, SINT32 len)
{
   SINT32 retval = FAILURE;
   tSIRF_MSG_SSB_TRKR_PEEKPOKE_CMD response;

   response.type = pMsg[0];

   if(response.type == 1) /* 4 byte poke response */
   {
       DebugPrint(("Poke acknowledgement received\n"));
       retval = ERROR_NONE;
   }
   else if (response.type == 0) /* 4-byte peek response */
   {
      response.address = (pMsg[1]<<24 | pMsg[2]<<16 | pMsg[3]<<8 | pMsg[4]);
      response.data    = (pMsg[5]<<24 | pMsg[6]<<16 | pMsg[7]<<8 | pMsg[8]);
      DebugPrint(("Peek response received. Address: 0x%08x, Data: 0x%08x\n", response.address, response.data));
      retval = ERROR_NONE;
   }

   return retval;
}

static SINT32 SSBmsgDeliver (MessageId mid, UINT8 * pMsg, SINT32 len)
{
    MessageId msgID;
    SINT32 returnStatus = ERROR_NONE_SSB_MSG_RCVD;
#if 0  
    UINT8 id; /* Used for debugging */
#endif
    msgID = (MessageId)*pMsg++;
    len--;

    switch (msgID) 
    {
    case MID_Ack:
        returnStatus = ERROR_ACK_RCVD;
        /*  DebugPrint("Ack Received\n"); */
        break;
    case MID_Nak:
        /*  DebugPrint("NAck Received\n"); */
        returnStatus = ERROR_NACK_RCVD;
        break;
    case MID_SWVersion:
        PollSwVersionHdlr(pMsg);
        break;
    case MID_RxMgrParams:
        /* DebugPrint("Response to Poll NAV Params Received\n"); */
        PollNavParamsResponseHdlr(pMsg,len);
        break;
    case MID_SWToolBox:     /* 0xB2 */
        {
           UINT8 msgSID;
           msgSID = *pMsg++;
           len--;

           switch(msgSID)
           {
           case SID_PM_PROMPT:   /* 0x90 */
              {
                 returnStatus = HandlePMPrompt(pMsg, len);
              }
              break;
           case SID_PM_ACK:      /* 0x91 */
              {
                 returnStatus = HandlePMAck(pMsg, len);
              }
              break;
           case SID_PEEKPOKE_RSP: /* 0x04 */
              {
                 returnStatus = HandlePeekPokeResponse(pMsg, len);
              }
              break;
           default:
               {
                  DebugPrint(("OSP:Unknown SW Tool Box Sub-Id: 0x%02X\n",msgSID));
               }
               break;
           }
        }
        break;
    default:
        break;
    }
#if 0
    /* Used for debugging */
    id = (UINT8)msgID;
    DebugPrint(("Rcvd MID:%d RtnStat:%d\n",id,returnStatus));
#endif
    return returnStatus;
}

/******************************************************************************
*  FUNCTION NAME: startSM2
*
*  Input:       Buffer pointer, message length, current temp offset in buffer
*
*  Output:      True/False
*
*  Description: State machine II if the first one fails
*
******************************************************************************/

static SINT32 startSM2 (UINT8 * pBuf, UINT16 msgLen, UINT16 bufferOffset)
{
    UINT16 len = 0, computedChecksum = 0, revievedChecksum = 0, i;
    SINT32 retval = FAILURE;
    SPMessageId msgId;

    /*
    * After the header A0 A2, the next 2 bytes are the length
    */
    len = pBuf[2];
    len = (len << 8) + pBuf[3];

    if (len > msgLen)
        return retval;

    /*
    * Read payload
    */
    for (i = 0; i < len; i++)
    {
        computedChecksum += pBuf[4 + i];
    }

    /*
    * After payload the next 2 bytes are checksum. The offset 4 is to account
    * for A0 A2, L1 L2(length of payload)
    */

    revievedChecksum = pBuf[4 + len];
    revievedChecksum = (revievedChecksum << 8) + pBuf[4 + len + 1];

    if ((revievedChecksum & SP_CKSUM_MASK) ==
        (computedChecksum & SP_CKSUM_MASK))
    {
        if ((SP_ETX0 == pBuf[len + 6]) && (SP_ETX1 == pBuf[len + 7]))
        {
            msgId = (SPMessageId) * (pBuf + 4);
            retval = SSBmsgDeliver ((MessageId) msgId, (pBuf + 4), len);
        }

    }
    else
    {
        retval = FAILURE;
    }

    return retval;
}


static SINT32 spSSBDeliver (UINT8 * pBuf, SINT32 bufLen)
{
    SPMessageId msgId;
    UINT16 i = 0, j = 0;
    SINT32 retval = ERROR_NONE;
    SSB_SPMsg.currentBufferOffset += bufLen;

    /* Temporary */
    PM_MemCpy(SSB_SPMsg.buf,pBuf,bufLen);

    for (i = 0; i < bufLen; i++)
    {
        switch (SSB_SPMsg.SSBMessageState)
        {
        case START1:
            {
                if (SP_STX0 == pBuf[i])
                {
                    SSB_SPMsg.SSBMessageState = START2;
                    SSB_SPMsg.msgStartOffset = 
                        SSB_SPMsg.currentBufferOffset - bufLen + i;
                    retval = ERROR_NONE;
                }
            }
            break;

        case START2:
            {
                if (SP_STX1 == pBuf[i])
                {
                    SSB_SPMsg.SSBMessageState = LENGTH;
                    SSB_SPMsg.rcvdLen = 0;
                    SSB_SPMsg.tempByteCount = 0;
                }
                else if (SP_STX0 == pBuf[i])
                {
                    SSB_SPMsg.msgStartOffset = 
                        SSB_SPMsg.currentBufferOffset - bufLen + i;
                    retval = ERROR_NONE;
                }
                else
                {
                    SSB_SPMsg.SSBMessageState = START1;
                    retval = FAILURE;
                }
            }
            break;

        case LENGTH:
            {
                SSB_SPMsg.rcvdLen = (SSB_SPMsg.rcvdLen << 8) + pBuf[i];
                if (++SSB_SPMsg.tempByteCount == 2)
                {
                    if (SSB_SPMsg.rcvdLen <
                        SSB_MAX_MSG_LEN + SP_TRANSPORT_CKS_LEN)
                    {
                        SSB_SPMsg.SSBMessageState = PAYLOAD;
                        SSB_SPMsg.tempByteCount = 0;
                        SSB_SPMsg.compChecksum = 0;
                    }
                    else
                    {
                        retval = FAILURE;
                    }
                }
            }
            break;

        case PAYLOAD:
            {
                SSB_SPMsg.compChecksum += pBuf[i];

                if (++SSB_SPMsg.tempByteCount == SSB_SPMsg.rcvdLen)
                {
                    SSB_SPMsg.tempByteCount = 0;
                    SSB_SPMsg.rcvdChecksum = 0;
                    SSB_SPMsg.SSBMessageState = CHECKSUM;
                }
                else
                {
                    /*
                    * We start a duplicate state machine here which tracks if there is
                    * a nested message.
                    */
                    j = i;
                    if (j < (bufLen - 1))
                    {
                        if ((SP_STX0 == pBuf[j]) && (SP_STX1 == pBuf[++j]))
                        {
                            if (startSM2 (&pBuf[i], bufLen, i) )
                            {
                                PM_MemSet(&SSB_SPMsg, 0, sizeof (SSB_SPMsg));
                                SSB_SPMsg.SSBMessageState = START1;
                            }
                        }
                    }

                }

            }
            break;

        case CHECKSUM:
            {
                /*
                * extract the two bytes checksum and compare it against computed one
                */
                SSB_SPMsg.rcvdChecksum =
                    (SSB_SPMsg.rcvdChecksum << 8) + pBuf[i];
                SSB_SPMsg.tempByteCount += 1;
                if ((SSB_SPMsg.tempByteCount == 2))
                {
                    if ((SSB_SPMsg.rcvdChecksum & SP_CKSUM_MASK) ==
                        (SSB_SPMsg.compChecksum & SP_CKSUM_MASK))
                    {
                        SSB_SPMsg.SSBMessageState = END1;
                    }
                    else
                    {
                        DebugPrint(("OSP:Checksum Error\n"));
                        retval = FAILURE;
                    }
                }

            }
            break;

        case END1:
            {
                if (SP_ETX0 == pBuf[i])
                {
                    SSB_SPMsg.SSBMessageState = END2;
                }
                else
                {
                    retval = FAILURE;
                }

            }
            break;

        case END2:
            {
                if ((SP_ETX1 == pBuf[i]) && (FAILURE != retval))
                {
                    msgId = (SPMessageId) SSB_SPMsg.buf[SSB_SPMsg.msgStartOffset + 4];
                    retval  = SSBmsgDeliver ((MessageId) msgId, 
                        &SSB_SPMsg.buf[SSB_SPMsg.msgStartOffset + 4],
                        SSB_SPMsg.rcvdLen);
                    PM_MemSet (&SSB_SPMsg, 0, sizeof (SSB_SPMsg));
                    SSB_SPMsg.SSBMessageState = START1;
                    SSB_SPMsg.msgStartOffset = 0;
                }
                else
                {
                    /*
                    * We start a duplicate state machine here which tracks if there is
                    * a nested message.
                    */
                    j = i;
                    if (j < (bufLen - 1))
                    {
                        if ((SP_STX0 == pBuf[j]) && (SP_STX1 == pBuf[++j]))
                        {
                            if (startSM2 (&pBuf[i], bufLen, i) )
                            {
                                PM_MemSet(&SSB_SPMsg, 0, sizeof (SSB_SPMsg));
                                SSB_SPMsg.SSBMessageState = START1;
                            }
                        }
                    }

                    retval = FAILURE;
                }
            }
            break;
        }
    }

    if (FAILURE == retval)
    {
        for (i = 2; i < bufLen; i++)
        {
            j = i;
            if ((SP_STX0 == pBuf[j]) && (SP_STX1 == pBuf[++j]))
            {
                if (startSM2 (&pBuf[i], bufLen, i) )
                {
                    PM_MemSet (&SSB_SPMsg, 0, sizeof (SSB_SPMsg));
                    SSB_SPMsg.SSBMessageState = START1;
                }
            }
        }

        DebugPrint( ("OSP: State machine recovery\n"));
        PM_MemSet (&SSB_SPMsg, 0, sizeof (SSB_SPMsg));
        SSB_SPMsg.SSBMessageState = START1;
        SSB_SPMsg.msgStartOffset = 0;
    }

    return retval;
}

static SINT32 ComputeChksum( UINT8* pBuf, SINT32 Cnt )
{
    SINT32 ChkSum;
    for (ChkSum = 0; Cnt > 0; Cnt--, pBuf++)
    {
        ChkSum += *pBuf;
    }
    return ChkSum;
}

/* Input Message Handling */
static unsigned short NTOS( unsigned short n )
{
    return ((n & 0xff) << 8) | ((n >> 8) & 0xff);
}

static unsigned short STON( unsigned short n )
{
    return NTOS( n );
}

static unsigned long NTOL( unsigned long n )
{ 
    return (n << 24)
        | (((n >> 8) & 0xff) << 16) 
        | (((n >> 16) & 0xff) << 8)
        | (n >> 24);
}

static unsigned long LTON( unsigned long n )
{ 
    return NTOL( n );
}
static void ImportUINT8(UINT8 *Dst, UINT8** pSrc)
{
    *Dst = **pSrc;
    *pSrc += 1;
}

static void ImportUINT16( WORD *Dst, UINT8** pSrc )
{
    *Dst = *((WORD*) *pSrc);
    *Dst = NTOS( *Dst );
    *pSrc += 2;
}

static void ImportINT16( short *Dst, UINT8** pSrc )
{
    *Dst = *((short*) *pSrc);
    *Dst = NTOS( *Dst );
    *pSrc += 2;
}

static void ImportUINT32( DWORD *Dst, UINT8** pSrc )
{
    *Dst = *((DWORD*) *pSrc);
    *Dst = NTOL( *Dst );
    *pSrc += 4;
}

static void ImportINT32( long *Dst, UINT8** pSrc )
{
    *Dst = *((long*) *pSrc);
    *Dst = NTOL( *Dst );
    *pSrc += 4;
}

static void RcvrParamSS2Import( UINT8* pSrc )
{
    UINT8 localuint8;
    INT16 localint16;
    UINT32 localuint32;
    INT32 localint32;

    /* ImportUINT8( &RcvrParamSS2.MsgID,                &pSrc ); */
    ImportUINT32( &localuint32,                 &pSrc );
    ImportUINT8 ( &localuint8,              &pSrc );
    ImportUINT8 ( &localuint8,               &pSrc );
    ImportINT16 ( &localint16,             &pSrc );
    ImportUINT8 ( &localuint8,         &pSrc );
    ImportUINT8 ( &localuint8,      &pSrc );
    ImportUINT8 ( &localuint8,           &pSrc );
    ImportUINT8 ( &localuint8,           &pSrc );
    ImportUINT8 ( &localuint8,           &pSrc );
    ImportUINT8 ( &localuint8,             &pSrc );
    ImportUINT32( &localuint32,                 &pSrc );
    ImportUINT8 ( &localuint8,          &pSrc );
    ImportINT16 ( &localint16,          &pSrc );
    ImportUINT8 ( &localuint8,           &pSrc );
    ImportUINT32( &localuint32,                 &pSrc );
    ImportUINT8 ( &localuint8,              &pSrc );
    ImportUINT8 ( &localuint8,             &pSrc );
    ImportUINT8 ( &localuint8,          &pSrc );
    ImportUINT32( &localuint32,                 &pSrc );
    ImportUINT8 ( &RcvrParamSS2.LP_PushToFix,         &pSrc );
    ImportINT32 ( &RcvrParamSS2.LP_OnTime,            &pSrc );
    ImportINT32 ( &RcvrParamSS2.LP_Interval,          &pSrc );
    ImportUINT8 ( &localuint8,  &pSrc );
    ImportINT32 ( &localint32,  &pSrc );
    ImportUINT8 ( &RcvrParamSS2.LP_PwrCyclingEnabled, &pSrc );
    ImportUINT32( &RcvrParamSS2.LP_MaxAcqTime,        &pSrc );
    ImportUINT32( &RcvrParamSS2.LP_MaxOffTime,        &pSrc );
    ImportUINT32( &localuint32,                 &pSrc );
    ImportUINT32( &localuint32,                 &pSrc );
}

static void PollNavParamsResponseHdlr(UINT8 * pMsg, SINT32 len)
{
    const char* strLP_State[]={"NotAvail","Continuous","PushToFix","TricklePwr","unk"};
    float LP_DutyCycle;
    char  LP_Str[300] = "";
    char  tmpStr[100] = "";

    RcvrParamSS2Import(pMsg);

    if( (!RcvrParamSS2.LP_OnTime) || (!RcvrParamSS2.LP_Interval) )
    {
        LP_DutyCycle = 0.0;
    }
    else
    {
        LP_DutyCycle = (float) (100 * RcvrParamSS2.LP_OnTime)
            / (float) (RcvrParamSS2.LP_Interval);
    }
    if( RcvrParamSS2.LP_PushToFix )
    {
        strcat( LP_Str, "Push-To-Fix enabled\n" );
    }
    else if( !RcvrParamSS2.LP_PwrCyclingEnabled )
    {
        strcat( LP_Str, "Continuous power enabled\n" );
    }
    else
    {
        sprintf( tmpStr, "TricklePower enabled with %ld ms on, %f%% duty cycle\n",
            RcvrParamSS2.LP_OnTime, LP_DutyCycle );
        strcat( LP_Str, tmpStr );
    }
    sprintf( tmpStr, "MaxAcqTime = %d ms; MaxOffTime = %d ms\n",
        RcvrParamSS2.LP_MaxAcqTime, RcvrParamSS2.LP_MaxOffTime );
    strcat( LP_Str, tmpStr );

    DebugPrint(("%s\n",LP_Str));

}

static SINT32 PollSwVersionHdlr(UINT8* pMsg)
{
    pMsg++; /* sirf version bytes */
    pMsg++; /* customer version bytes */
    PM_strcpy(SwVersion, pMsg);

    DebugPrint(("SW Version is %s\n", SwVersion));
    return ERROR_NONE;
}